### MANAGED BY PUPPET ###

<% %w(mysql client mysqldump mysqladmin mysqlcheck).each do |section| %>
[<%= section -%>]
user=root
host=localhost
<% unless scope.lookupvar('mysql::server::root_password') == 'UNSET' -%>
password='<%= scope.lookupvar('mysql::server::root_password') %>'
<% end -%>
socket=<%= @options['client']['socket'] %>
<% end %>
